@@ -39,6 +39,8 @@
 	(G_TYPE_INSTANCE_GET_PRIVATE \
 	((obj), CAMEL_TYPE_IMAPX_CONN_MANAGER, CamelIMAPXConnManagerPrivate))
 
+static CamelIMAPXServer *imapx_conn_manager_get_connection (CamelIMAPXConnManager *con_man, const gchar *folder_name, GCancellable *cancellable, GError **error);
+
 typedef struct _ConnectionInfo ConnectionInfo;
 
 struct _CamelIMAPXConnManagerPrivate {
@@ -384,6 +386,8 @@ camel_imapx_conn_manager_class_init (CamelIMAPXConnManagerClass *class)
 	object_class->dispose = imapx_conn_manager_dispose;
 	object_class->finalize = imapx_conn_manager_finalize;
 
+	class->get_connection = imapx_conn_manager_get_connection;
+
 	g_object_class_install_property (
 		object_class,
 		PROP_STORE,
@@ -639,8 +643,8 @@ camel_imapx_conn_manager_get_store (CamelIMAPXConnManager *con_man)
 	return CAMEL_STORE (con_man->priv->store);
 }
 
-CamelIMAPXServer *
-camel_imapx_conn_manager_get_connection (CamelIMAPXConnManager *con_man,
+static CamelIMAPXServer *
+imapx_conn_manager_get_connection (CamelIMAPXConnManager *con_man,
                                          const gchar *folder_name,
                                          GCancellable *cancellable,
                                          GError **error)
@@ -727,6 +731,28 @@ camel_imapx_conn_manager_close_connections (CamelIMAPXConnManager *con_man)
 	CON_WRITE_UNLOCK (con_man);
 }
 
+/*----------------------------------------------------------------------------*/
+/* virtualized functions */
+
+CamelIMAPXServer*
+camel_imapx_conn_manager_get_connection (CamelIMAPXConnManager *self,
+                                         const gchar *folder_name,
+                                         GCancellable *cancellable,
+                                         GError **error)
+{
+	CamelIMAPXServer *server = NULL;
+	CamelIMAPXConnManagerClass *klass = NULL;
+
+	g_return_val_if_fail (CAMEL_IS_IMAPX_CONN_MANAGER (self), NULL);
+
+	klass = CAMEL_IMAPX_CONN_MANAGER_GET_CLASS (self);
+	server = klass->get_connection (self,
+	                                folder_name,
+	                                cancellable,
+	                                error);
+	return server;
+}
+
 /*----------------------------------------------------------------------------*/
 /* "friend" API for subclasses */
 
@@ -737,7 +763,7 @@ camel_imapx_conn_manager_find_connection_unlocked (CamelIMAPXConnManager *con_ma
                                                    const gchar *folder_name)
 {
 	g_assert (CAMEL_IS_IMAPX_CONN_MANAGER (con_man));
-	g_assert (folder_name != NULL);
+	/* folder_name may be NULL */
 
 	return imapx_find_connection_unlocked (con_man, folder_name);
 }
