@@ -39,6 +39,10 @@
 
 /*----------------------------------------------------------------------------*/
 
+#include <config.h>
+
+#include <glib/gi18n-lib.h>
+
 #include <libekolabconv/main/src/kolab-conv.h>
 
 #include <libekolabutil/kolab-util-glib.h>
@@ -254,8 +258,8 @@ kolab_mail_side_cache_sql_query_object (KolabUtilSqliteDb *kdb,
 		g_set_error (err,
 		             KOLAB_BACKEND_ERROR,
 		             KOLAB_BACKEND_ERROR_DATATYPE_KOLAB,
-		             "%s: UID (%s) Folder (%s) has zero mail parts",
-		             __func__, uid, foldername);
+		             _("Internal inconsistency detected: Invalid cache object with zero mail message parts, UID '%s', Folder '%s'"),
+		             uid, foldername);
 		g_free (tblname);
 		return NULL;
 	}
@@ -287,8 +291,8 @@ kolab_mail_side_cache_sql_query_object (KolabUtilSqliteDb *kdb,
 				g_set_error (&tmp_err,
 				             KOLAB_UTIL_ERROR,
 				             KOLAB_UTIL_ERROR_SQLITE_DB,
-				             "%s: %s",
-				             __func__, sqlite3_errmsg (kdb->db));
+				             _("SQLite Error: %s"),
+				             sqlite3_errmsg (kdb->db));
 			}
 			break;
 		}
@@ -307,8 +311,8 @@ kolab_mail_side_cache_sql_query_object (KolabUtilSqliteDb *kdb,
 			g_set_error (&tmp_err,
 			             KOLAB_UTIL_ERROR,
 			             KOLAB_UTIL_ERROR_SQLITE_DB,
-			             "%s: UID (%s) Folder (%s) expected (%i) payload bytes, got (%i)",
-			             __func__, uid, foldername, mpart->length, nbytes);
+			             _("Internal inconsistency detected: Invalid cache object has %i payload bytes, expected %i bytes, UID '%s', Folder '%s'"),
+			             mpart->length, nbytes, uid, foldername);
 			break;
 		}
 		mpart->name = g_strdup ((gchar *) sqlite3_column_text (sql_stmt,
@@ -418,8 +422,8 @@ kolab_mail_side_cache_sql_store_object (KolabUtilSqliteDb *kdb,
 		g_set_error (&tmp_err,
 		             KOLAB_UTIL_ERROR,
 		             KOLAB_UTIL_ERROR_SQLITE_DB,
-		             "%s: %s",
-		             __func__, sqlite3_errmsg (kdb->db));
+		             _("SQLite Error: %s"),
+		             sqlite3_errmsg (kdb->db));
 		(void)kolab_util_sqlite_fnlz_stmt (kdb, sql_stmt, NULL);
 		break;
 	}
@@ -678,8 +682,7 @@ kolab_mail_side_cache_store (KolabMailSideCache *self,
 		g_set_error (err,
 		             KOLAB_BACKEND_ERROR,
 		             KOLAB_BACKEND_ERROR_INTERNAL,
-		             "%s: Kolab UID not set on handle!",
-		             __func__);
+		             _("Internal inconsistency detected: PIM Object handle has no Kolab UID set"));
 		return FALSE;
 	}
 
@@ -725,8 +728,7 @@ kolab_mail_side_cache_retrieve (KolabMailSideCache *self,
 		g_set_error (err,
 		             KOLAB_BACKEND_ERROR,
 		             KOLAB_BACKEND_ERROR_INTERNAL,
-		             "%s: Kolab UID not set on handle!",
-		             __func__);
+		             _("Internal inconsistency detected: PIM Object handle has no Kolab UID set"));
 		return FALSE;
 	}
 
@@ -735,8 +737,8 @@ kolab_mail_side_cache_retrieve (KolabMailSideCache *self,
 		g_set_error (err,
 		             KOLAB_BACKEND_ERROR,
 		             KOLAB_BACKEND_ERROR_INTERNAL,
-		             "%s: UID (%s) foldername not set",
-		             __func__, kolab_uid);
+		             _("Internal inconsistency detected: Folder name not set on PIM Object handle, UID '%s'"),
+		             kolab_uid);
 		return FALSE;
 	}
 
@@ -752,8 +754,8 @@ kolab_mail_side_cache_retrieve (KolabMailSideCache *self,
 		g_set_error (err,
 		             KOLAB_BACKEND_ERROR,
 		             KOLAB_BACKEND_ERROR_NOTFOUND,
-		             "%s: could not find Kolab UID (%s) in folder (%s)",
-		             __func__, kolab_uid, foldername);
+		             _("Could not find cache object, UID '%s', Folder '%s'"),
+		             kolab_uid, foldername);
 		return FALSE;
 	}
 
@@ -819,8 +821,7 @@ kolab_mail_side_cache_delete (KolabMailSideCache *self,
 		g_set_error (err,
 		             KOLAB_BACKEND_ERROR,
 		             KOLAB_BACKEND_ERROR_INTERNAL,
-		             "%s: Kolab UID not set on handle!",
-		             __func__);
+		             _("Internal inconsistency detected: PIM Object handle has no Kolab UID set"));
 		return FALSE;
 	}
 
@@ -829,8 +830,8 @@ kolab_mail_side_cache_delete (KolabMailSideCache *self,
 		g_set_error (err,
 		             KOLAB_BACKEND_ERROR,
 		             KOLAB_BACKEND_ERROR_INTERNAL,
-		             "%s: UID (%s) foldername not set",
-		             __func__, kolab_uid);
+		             _("Internal inconsistency detected: Folder name not set on PIM Object handle, UID '%s'"),
+		             kolab_uid);
 		return FALSE;
 	}
 
