@@ -47,6 +47,7 @@ static gchar*
 get_vtimezone (const ECalComponentWithTZ *ectz)
 {
 	icalcomponent *icc = NULL;
+	icalcomponent *cloned_icc = NULL;
 	icalproperty *dtstamp = NULL;
 	icalproperty *uid = NULL;
 	gchar *ical_str = NULL;
@@ -55,16 +56,18 @@ get_vtimezone (const ECalComponentWithTZ *ectz)
 		return NULL;
 
 	icc = e_cal_component_get_icalcomponent (ectz->timezone);
+	cloned_icc = icalcomponent_new_clone (icc);
 
 	/* uid and dtstamp are not needed (nor wanted) in timezone block */
-	uid = icalcomponent_get_first_property(icc, ICAL_UID_PROPERTY);
-	icalcomponent_remove_property (icc, uid);
+	uid = icalcomponent_get_first_property(cloned_icc, ICAL_UID_PROPERTY);
+	icalcomponent_remove_property (cloned_icc, uid);
 	free(uid);
-	dtstamp = icalcomponent_get_first_property(icc, ICAL_DTSTAMP_PROPERTY);
-	icalcomponent_remove_property (icc, dtstamp);
+	dtstamp = icalcomponent_get_first_property(cloned_icc, ICAL_DTSTAMP_PROPERTY);
+	icalcomponent_remove_property (cloned_icc, dtstamp);
 	free(dtstamp);
 
-	ical_str = icalcomponent_as_ical_string (icc);
+	ical_str = icalcomponent_as_ical_string (cloned_icc);
+	icalcomponent_free (cloned_icc);
 
 	return ical_str;
 }
